import{ world, system, ItemStack} from "@minecraft/server";

const tall_door_list = [ 
	"kd:acacia_tall_door_item", 
	"kd:cherry_tall_door_item", 
	"kd:bamboo_tall_door_item", 
	"kd:birch_tall_door_item", 
	"kd:crimson_tall_door_item", 
	"kd:dark_oak_tall_door_item", 
	"kd:jungle_tall_door_item", 
	"kd:mangrove_tall_door_item", 
	"kd:oak_tall_door_item", 
	"kd:quartz_tall_door_item", 
	"kd:spruce_tall_door_item", 
	"kd:stone_tall_door_item", 
	"kd:warped_tall_door_item", 
	"kd:iron_bars_tall_door_item"
]

const door_list = [ 
	"kd:acacia_chiseled_door_item", 
	"kd:cherry_chiseled_door_item", 
	"kd:bamboo_chiseled_door_item", 
	"kd:birch_chiseled_door_item", 
	"kd:crimson_chiseled_door_item", 
	"kd:dark_oak_chiseled_door_item", 
	"kd:jungle_chiseled_door_item", 
	"kd:mangrove_chiseled_door_item", 
	"kd:oak_chiseled_door_item", 
	"kd:iron_bars_chiseled_door_item", 
	"kd:iron_bars_door_item", 
	"kd:quartz_chiseled_door_item", 
	"kd:quartz_door_item", 
	"kd:spruce_chiseled_door_item", 
	"kd:stone_chiseled_door_item", 
	"kd:stone_door_item", 
	"kd:warped_chiseled_door_item", 
	"kd:door_template_item",
	"kd:acacia_lited_door_item", 
	"kd:cherry_lited_door_item", 
	"kd:bamboo_lited_door_item", 
	"kd:birch_lited_door_item", 
	"kd:crimson_lited_door_item", 
	"kd:dark_oak_lited_door_item", 
	"kd:jungle_lited_door_item", 
	"kd:mangrove_lited_door_item", 
	"kd:oak_lited_door_item", 
	"kd:vintage_elevator_door_item", 
	"kd:quartz_lited_door_item", 
	"kd:spruce_lited_door_item", 
	"kd:stone_lited_door_item", 
	"kd:warped_lited_door_item",
	"kd:acacia_sliding_door_item", 
	"kd:cherry_sliding_door_item", 
	"kd:bamboo_sliding_door_item", 
	"kd:birch_sliding_door_item", 
	"kd:crimson_sliding_door_item", 
	"kd:dark_oak_sliding_door_item", 
	"kd:jungle_sliding_door_item", 
	"kd:mangrove_sliding_door_item", 
	"kd:oak_sliding_door_item", 
	"kd:iron_bars_sliding_door_item", 
	"kd:iron_bars_door_item", 
	"kd:quartz_sliding_door_item", 
	"kd:quartz_door_item", 
	"kd:spruce_sliding_door_item", 
	"kd:stone_sliding_door_item", 
	"kd:stone_door_item", 
	"kd:warped_sliding_door_item"
]
const dimensions = [
	world.getDimension("minecraft:the_end"),
	world.getDimension("minecraft:nether"),
	world.getDimension("minecraft:overworld")
];

function offset(loc, dx, dy, dz){
	return { x: loc.x+dx, y: loc.y+dy, z: loc.z+dz};
};

let offset_position = [
	{ x: 0, y: 1, z: 0 },
	{ x: 0, y: -1, z: 0 },
	{ x: 1, y: 0, z: 0 },
	{ x: -1, y: 0, z: 0 },
	{ x: 0, y: 0, z: 1 },
	{ x: 0, y: 0, z: -1 },
]

function getRedstonePower(block, repeat = true){
	if(block.isAir || block.isLiquid) return undefined;
	let permutation = block.permutation;
	let states = permutation.getAllStates();
	
	if(states["door:door_hinge_bit"] != undefined || states.door_hinge_bit != undefined){
		return undefined;
	}else if(states.redstone_signal != undefined){
		return states.redstone_signal;
	}else if(states.button_pressed_bit){
		return 16;
	}else if(states.lever_direction != undefined && states.open_bit){
		return 16;
	}else if(states.powered_bit){
		return 16;
	}else if(permutation.matches("minecraft:redstone_torch") || permutation.matches("minecraft:redstone_block") || permutation.matches("minecraft:powered_repeater") || permutation.matches("minecraft:powered_comparator")){
		return 16;
	}
	if(repeat){
		for(let location of offset_position){
			let power = getRedstonePower(block.offset(location), false);
			if(power != undefined){
				return power;
			}
		}
	}
}

world.beforeEvents.itemUseOn.subscribe(s => {
	if(door_list.includes(s.itemStack.typeId) && s.blockFace == "Up"){
		// let block_scan = s.block.location;
		s.cancel = !s.block.above(2).isAir;
	}else if(tall_door_list.includes(s.itemStack.typeId) && s.blockFace == "Up"){
		// let block_scan = s.block.location;
		// let block_scan_top = s.block.location;
		// block_scan_top.y += 3;
		// block_scan.y += 2;
		s.cancel = !s.block.above(2).isAir || !s.block.above(3).isAir;
	}
});

world.afterEvents.itemUseOn.subscribe(s => {
	if(s.itemStack.typeId == "kd:lock_item" && s.source.isSneaking){
		let get_source = s.source;
		let top_loc = s.block.location;
		let door_hinge_name = (get_source.dimension.getBlock(top_loc).hasTag("door")) ? "door:door_hinge_bit" : "door_hinge_bit";
		
		if(get_source.dimension.getBlock(top_loc).permutation.getState(door_hinge_name) != undefined){
			let parent_loc = s.block.location;
			if(s.block.hasTag("sliding_door_placeholder")){
				let off = (s.block.permutation.getState("door:door_hinge_bit")) ? 1 : -1;
				switch(s.block.permutation.getState("door:direction")){
					case 0:
						parent_loc.z -= off;
						top_loc.z -= off;
					break;
					case 1:
						parent_loc.x += off;
						top_loc.x += off;
					break;
					case 2:
						parent_loc.z += off;
						top_loc.z += off;
					break;
					case 3:
						parent_loc.x -= off;
						top_loc.x -= off;
					break;
				}
			}

			let lock = get_source.dimension.spawnEntity("kd:lock", offset(parent_loc, 0.5, 0, 0.5));
			let lock_blocking = [];

			let upper_block_name = (get_source.dimension.getBlock(parent_loc).hasTag("door")) ? "door:upper_block_bit" : "upper_block_bit";
			let direction_name = (get_source.dimension.getBlock(parent_loc).hasTag("door")) ? "door:direction" : "direction";
			lock_blocking.push(lock);
			
			while(!get_source.dimension.getBlock(parent_loc).permutation.getState(upper_block_name) == false){
				parent_loc.y -= 1;
				let entity = get_source.dimension.spawnEntity("kd:lock", offset(parent_loc, 0.5, 0, 0.5));
				lock_blocking.push(entity);
			}
			top_loc.y += 1;
			while(get_source.dimension.getBlock(top_loc).permutation.getState(upper_block_name) != undefined && !get_source.dimension.getBlock(top_loc).permutation.getState(upper_block_name) == false){
				let entity = get_source.dimension.spawnEntity("kd:lock", offset(top_loc, 0.5, 0, 0.5));
				lock_blocking.push(entity);
				top_loc.y += 1;
			}
			top_loc.y -= 1;
			
			let get_door_dir = get_source.dimension.getBlock(parent_loc).permutation.getState(direction_name);
			let child_loc = { x: parent_loc.x, y: parent_loc.y, z: parent_loc.z};
			let param_loc = { x: parent_loc.x, y: parent_loc.y+1, z: parent_loc.z};
			let left_scan = { x: parent_loc.x, y: parent_loc.y, z: parent_loc.z};
			let right_scan = { x: parent_loc.x, y: parent_loc.y, z: parent_loc.z};
			
			left_scan.y += 1;
			right_scan.y += 1;
			
			switch(get_door_dir){
				case 0:
					left_scan.z -= 1;
					right_scan.z += 1;
				break;
				case 1:
					left_scan.x += 1;
					right_scan.x -= 1;
				break;
				case 2:
					left_scan.z += 1;
					right_scan.z -= 1;
				break;
				case 3:
					left_scan.x -= 1;
					right_scan.x += 1;
				break;
			}
			
			let left_scan_dir = { x: left_scan.x, y: left_scan.y-1, z: left_scan.z};
			let right_scan_dir = { x: right_scan.x, y: right_scan.y-1, z: right_scan.z};
			
			let first_pos = parent_loc;
			
			let left_scan_block = get_source.dimension.getBlock(left_scan);
			let right_scan_block = get_source.dimension.getBlock(right_scan);

			if(!right_scan_block.hasTag("sliding_door_placeholder") && right_scan_block.permutation.getState(door_hinge_name) != undefined && right_scan_block.permutation.getState(upper_block_name) && get_source.dimension.getBlock(right_scan_dir).permutation.getState(direction_name) == get_door_dir && (get_source.dimension.getBlock(param_loc).permutation.getState(upper_block_name) == true && get_source.dimension.getBlock(param_loc).permutation.getState(door_hinge_name) == false) && (right_scan_block.permutation.getState(upper_block_name) == true && right_scan_block.permutation.getState(door_hinge_name) == true)){
				child_loc.x = right_scan.x;
				child_loc.z = right_scan.z;
			}else if(!left_scan_block.hasTag("sliding_door_placeholder") && left_scan_block.permutation.getState(door_hinge_name) != undefined && left_scan_block.permutation.getState(upper_block_name) && get_source.dimension.getBlock(left_scan_dir).permutation.getState(direction_name) == get_door_dir && (get_source.dimension.getBlock(param_loc).permutation.getState(upper_block_name) == true && get_source.dimension.getBlock(param_loc).permutation.getState(door_hinge_name) == true) && (left_scan_block.permutation.getState(upper_block_name) == true && left_scan_block.permutation.getState(door_hinge_name) == false)){
				child_loc.x = left_scan.x;
				child_loc.z = left_scan.z;
			}
		
			
			if(!(child_loc.x == parent_loc.x && child_loc.y == parent_loc.y && child_loc.z == parent_loc.z) && !get_source.dimension.getBlock(child_loc).permutation.getState(upper_block_name)){
				let entity = get_source.dimension.spawnEntity("kd:lock", offset(child_loc, 0.5, 0, 0.5));
				lock_blocking.push(entity);
				child_loc.y += 1;
				while(get_source.dimension.getBlock(child_loc).permutation.getState(upper_block_name) != undefined && !get_source.dimension.getBlock(child_loc).permutation.getState(upper_block_name) == false){
					let entity = get_source.dimension.spawnEntity("kd:lock", offset(child_loc, 0.5, 0, 0.5));
					lock_blocking.push(entity);
					child_loc.y += 1;
				}
				child_loc.y -= 1;
			}else{
				child_loc = top_loc;
			}
			
			let mid_x = (parent_loc.x+child_loc.x)/2+0.5;
			let mid_y = (parent_loc.y+child_loc.y)/2;
			let mid_z = (parent_loc.z+child_loc.z)/2+0.5;
			
			let lock_icon = get_source.dimension.spawnEntity("kd:lock", { x: (parent_loc.x+child_loc.x)/2, y: (parent_loc.y+child_loc.y)/2, z: (parent_loc.z+child_loc.z)/2});
			
			let lock_id = "LOCK" + lock_icon.id;
			lock_icon.addTag(lock_id);
			lock_icon.getComponent("minecraft:mark_variant").value = get_door_dir;
			lock_icon.setProperty("lock:inverted", (get_door_dir == 2 && s.blockFace == "West") || (get_door_dir == 0 && s.blockFace == "East") || (get_door_dir == 1 && s.blockFace == "South") || (get_door_dir == 3 && s.blockFace == "North"));
			lock_icon.teleport({ x: mid_x, y: mid_y, z: mid_z });
			lock_icon.triggerEvent("set_icon");
			for(let entity of lock_blocking){
				entity.addTag(lock_id);
			}
			
			let key = new ItemStack("kd:lock_key");
			key.setLore( [ "\n§r§9Lock ID: " + lock_id ] );
			get_source.getComponent("minecraft:equippable").setEquipment("Mainhand", key);
			
		}
	}
});

world.afterEvents.entityHitEntity.subscribe(s => {
	if(s.hitEntity && s.hitEntity.typeId == "kd:lock"){
		let lock_id = s.hitEntity.getTags()[0];
		let lock_display = "Lock ID: " + lock_id;
		let player_name = s.damagingEntity.nameTag;
		let item = s.damagingEntity.getComponent("minecraft:equippable").getEquipment("Mainhand");
		if(item && item.typeId == "kd:lock_key" && item.getLore()[0] == "\n§r§9" + lock_display){
			let lock = new ItemStack("kd:lock_item");
			s.damagingEntity.getComponent("minecraft:equippable").setEquipment("Mainhand", lock);
			s.hitEntity.runCommandAsync("playsound random.metal.door_open @a ~ ~ ~ 1 2");
			s.damagingEntity.runCommandAsync("event entity @e[type=kd:lock, tag=" + lock_id + "] despawn");
		}else if(item && item.typeId == "kd:lock_pick"){
			let amount = item.amount;
			amount -= Array.from(world.getPlayers( { excludeGameModes: [ "creative" ], name: player_name})).length;
			if(amount > 0){
				item.amount = amount;
			}else{
				item = undefined;
			}

			s.damagingEntity.getComponent("minecraft:equippable").setEquipment("Mainhand", item);
			s.hitEntity.triggerEvent("use_lock_pick");
			s.hitEntity.runCommandAsync("playsound random.break @a ~ ~ ~ 0.5 3");
		}else{
			s.damagingEntity.onScreenDisplay.setActionBar(lock_display);
		}
	}
	
}, { entityTypes: [ "minecraft:player" ] });

world.afterEvents.dataDrivenEntityTrigger.subscribe(s => {
	let entity;
	let error = 0;
	try{
		entity = s.entity;
		if(entity.getVelocity().x != 0 || entity.getVelocity().y != 0 || entity.getVelocity().z != 0){
			error++;
		}
	}catch(err){
		return;
	}
	

	if(s.eventId == "kd:unlock" || error > 0){
		entity.runCommandAsync("event entity @e[type=kd:lock, tag=" + entity.getTags()[0] + "] despawn");
		
	}
}, { entityTypes: [ "kd:lock" ] });

world.afterEvents.playerBreakBlock.subscribe(s => {
	if(s.brokenBlockPermutation.getState("door:upper_block_bit") == 2){
		let block_scan = s.block.location;
		block_scan.y -= 1;
		// s.block.dimension.getBlock(block_scan).setType("minecraft:air");
		s.block.dimension.runCommand("setblock " + block_scan.x + " "  + block_scan.y + " " + block_scan.z + " minecraft:air replace");
	}
	
	if(s.brokenBlockPermutation.getState("door:placeholder") != undefined){
		let off = (s.brokenBlockPermutation.getState("door:door_hinge_bit")) ? 1 : -1;
		let scan = {x: 0, y: 1, z: 0};
		switch(s.brokenBlockPermutation.getState("door:direction")){
			case 0:
				scan.z -= off;
			break;
			case 1:
				scan.x += off;
			break;
			case 2:
				scan.z += off;
			break;
			case 3:
				scan.x -= off;
			break;
		}
		if(s.block.below().permutation.getState("door:placeholder") != undefined) scan.y -= 1;
		// s.block.offset(scan).setType("minecraft:air");
		scan = offset(s.block.location, scan.x, scan.y, scan.z);
		s.block.dimension.runCommand("setblock " + scan.x + " "  + scan.y + " " + scan.z + " minecraft:air replace");
	}
});


function blockInteractProp(blockWorld, loc){
	let block = blockWorld.getBlock(loc);
	let child_door = block.permutation.withState("door:interacted", true);
	block.setPermutation(child_door);
}

function blockSetProp(blockWorld, loc, replace){
	let block = blockWorld.getBlock(loc);
	let child_door = block.permutation.withState("door:open_bit", replace);
	block.setPermutation(child_door);
}

function blockSetPropVanilla(blockWorld, loc, replace){
	let block = blockWorld.getBlock(loc);
	let child_door = block.permutation.withState("open_bit", replace);
	block.setPermutation(child_door);
}

function door_sound(blockPar){
	let soundType  = "wood";
	if(blockPar.hasTag("metal")){
		soundType = "metal";
	}else if(blockPar.hasTag("bamboo")){
		soundType = "bamboo";
	}else if(blockPar.hasTag("stone") || blockPar.hasTag("nether")){
		soundType = "heavy";
	}
	
	if(!blockPar.permutation.getState("door:open_bit")){
		blockPar.dimension.runCommandAsync("playsound random." + soundType + ".door_open @a " + blockPar.location.x + " " + + blockPar.location.y + " " + + blockPar.location.z);
	}else{
		blockPar.dimension.runCommandAsync("playsound random." + soundType + ".door_close @a " + blockPar.location.x + " " + + blockPar.location.y + " " + + blockPar.location.z);
	}
}

function interact_door(blockPar){
	door_sound(blockPar);
	let dim = blockPar.dimension;
	let blockPos = blockPar.location;
	let condition = !blockPar.permutation.getState("door:open_bit");
	blockSetProp(dim, blockPos, condition);
	let go_up = 1
	
	while(blockPar.dimension.getBlock(offset(blockPos, 0, go_up, 0)).hasTag("door") && blockPar.dimension.getBlock(offset(blockPos, 0, go_up, 0)).permutation.getState("door:upper_block_bit") > 0){
		let offsetB = offset(blockPos, 0, go_up, 0)
		blockSetProp(dim, offsetB, condition);
		go_up += 1;
	}
	let go_down = 0;
	while(blockPar.dimension.getBlock(offset(blockPos, 0, go_down, 0)).hasTag("door") && blockPar.dimension.getBlock(offset(blockPos, 0, go_down, 0)).permutation.getState("door:upper_block_bit") > 0){
		go_down -= 1;
		let offsetB = offset(blockPos, 0, go_down, 0)
		blockSetProp(dim, offsetB, condition);
	}
}

system.afterEvents.scriptEventReceive.subscribe(s => {
	let block = s.sourceBlock;
	let block_powered = false;

	let scan = {x: 0, y: 0, z: 0};
	let off_block = [];


	if(block.hasTag("sliding_door")){
		let off = (block.permutation.getState("door:door_hinge_bit")) ? -1 : 1;
		switch(block.permutation.getState("door:direction")){
			case 0:
				scan.z -= off;
			break;
			case 1:
				scan.x += off;
			break;
			case 2:
				scan.z += off;
			break;
			case 3:
				scan.x -= off;
			break;
		}
		let offset_block = block.offset(scan);
		if(offset_block != undefined && offset_block.isAir == false && block.permutation.getState("door:open_bit") == false) return;
		off_block.push(offset_block);
		
		try{
			let offset = {x: 0, y: 0, z: 0};
			offset.y = (block.permutation.getState("door:upper_block_bit")) ? -1 : 1;
			offset_block = offset_block.offset(offset);
			if(offset_block != undefined && offset_block.isAir == false && block.permutation.getState("door:open_bit") == false) return;
			off_block.push(offset_block);
		}catch(err){}
	}

    if(s.message == "bottom_check"){
		
		let blockPar = s.sourceBlock;
		let blockPos = blockPar.location;

		if(blockPar.below().isAir && blockPar.permutation.getState("door:upper_block_bit") == 0){
			// blockPar.dimension.getBlock(offset(blockPos, 0, 1, 0)).setType("minecraft:air");
			
			blockPar.dimension.runCommand("setblock " + blockPos.x + " "  + (blockPos.y + 1) + " " + blockPos.z + " minecraft:air replace");
		}
		
		if(blockPar.permutation.getState("door:powered") != undefined){
			block_powered = true;
			let dim = blockPar.dimension;
			let get_lock = Array.from(blockPar.dimension.getEntitiesAtBlockLocation(blockPos));
			let has_locked = false;
			for(let entity_in_door of get_lock){
				if(entity_in_door.typeId == "kd:lock"){
					has_locked = true;
					break;
				}
			}
			if(!has_locked){
				let power;
				try{
					power = getRedstonePower(blockPar.below()) > 0;
				}catch(err){
					return;
				}
				let go_up = 0;
				while(blockPar.above(go_up).hasTag("door") && (blockPar.above(go_up).permutation.getState("door:upper_block_bit") > 0 || go_up == 0) && !power){
					try{
						if(getRedstonePower(blockPar.dimension.getBlock(offset(blockPos, 0, go_up, 1))) > 0){
							power = true;
						}else if(getRedstonePower(blockPar.dimension.getBlock(offset(blockPos, 0, go_up, -1))) > 0 && !power){
							power = true
						}else if(getRedstonePower(blockPar.dimension.getBlock(offset(blockPos, 1, go_up, 0))) > 0 && !power){
							power = true
						}else if(getRedstonePower(blockPar.dimension.getBlock(offset(blockPos, -1, go_up, 0))) > 0 && !power){
							power = true
						}
						if(power) break;
						go_up++;
					}catch(err){
						break;
					}
					
				}
				
				if(power != blockPar.permutation.getState("door:powered")){
					door_sound(blockPar);
					let condition = power;

					let child_door = blockPar.dimension.getBlock(blockPar.location).permutation.withState("door:open_bit", condition).withState("door:powered", power ? 1 : 0);
					blockPar.setPermutation(child_door);
					
					
					let go_down = 0;
					let go_up = 1;
					while(blockPar.above(go_down).hasTag("door") && blockPar.above(go_down).permutation.getState("door:upper_block_bit") > 0){
						go_down -= 1;
						let offsetB = offset(blockPos, 0, go_down, 0);
						blockSetProp(dim, offsetB, condition);
					}
					while(blockPar.above(go_up).hasTag("door") && blockPar.above(go_up).permutation.getState("door:upper_block_bit") > 0){
						let offsetB = offset(blockPos, 0, go_up, 0);
						blockSetProp(dim, offsetB, condition);
						go_up += 1;
					}
				}
			}
		}
	}else if(s.message == "interact_stable"){
		let blockPar = s.sourceBlock;
		let blockPos = blockPar.location;
		interact_door(blockPar);
	}else if(s.message == "interact"){
		let blockPar = s.sourceBlock;
		let blockPos = blockPar.location;
		let ancor = 0;
		while(blockPar.above(ancor).hasTag("door") && blockPar.above(ancor).permutation.getState("door:upper_block_bit") > 0){
			ancor -= 1;
		}
		interact_door(blockPar.above(ancor));
	}
	if(block.hasTag("sliding_door") && (s.message == "interact_stable" || s.message == "interact" || block_powered) && block.permutation.getState("door:open_bit")){
		let dimension = block.dimension;
		for(let replace of off_block){
			let loc = replace.location;
			let parent_state = block.permutation;

			dimension.runCommandAsync("setblock " + loc.x + " " + loc.y + " " + loc.z + " kd:sliding_door_placeholder [\"door:direction\": " + parent_state.getState("door:direction") + ", \"door:door_hinge_bit\": " + parent_state.getState("door:door_hinge_bit") + "]");
		}
	}

	if(s.message == "interact_slider"){
		let off = (block.permutation.getState("door:door_hinge_bit")) ? 1 : -1;
		let scan = block.location;
		switch(block.permutation.getState("door:direction")){
			case 0:
				scan.z -= off;
			break;
			case 1:
				scan.x += off;
			break;
			case 2:
				scan.z += off;
			break;
			case 3:
				scan.x -= off;
			break;
		}
		blockInteractProp(block.dimension, scan);
	}
}, { namespaces: [ "door" ]}); 

system.runInterval(() => {
	// for(let playerData of world.getPlayers()){
	// 	let block = playerData.getBlockFromViewDirection().block;
	// 	console.warn(JSON.stringify(block.permutation.getAllStates()))
	// }
	for(let dimension of dimensions){
		for(let entity of dimension.getEntities({type: "kd:lock"})){
			let get_block = dimension.getBlock({ x: Math.floor(entity.location.x), y: Math.floor(entity.location.y), z: Math.floor(entity.location.z)});
			if(get_block && (get_block.permutation.getState("door_hinge_bit") != undefined || get_block.permutation.getState("door:door_hinge_bit") != undefined)){
				let setFunction = get_block.permutation.getState("door_hinge_bit") != undefined ? blockSetPropVanilla : blockSetProp;
				setFunction(dimension, { x: Math.floor(entity.location.x), y: Math.floor(entity.location.y), z: Math.floor(entity.location.z)}, false);
			}
		}
	}
}); 
/*
system.afterEvents.scriptEventReceive.subscribe(s => {
    if(s.message == "target"){
		s.sourceEntity.addTag("use_duplicator");
	}else if(s.message == "duplicate"){
		let lock_id = Array.from(s.sourceBlock.dimension.getEntitiesAtlocation(s.sourceBlock.location))[0];
		let lock_container = lock_id.getComponent("minecraft:inventory").container;
		let lockID = lock_container.getSlot(0).getLore()[0];
		let otherID = lock_container.getSlot(1).getLore();
		otherID[0] = lockID;
		lock_container.getSlot(1).setLore(otherID);
		s.sourceBlock.dimension.spawnItem(lock_container.getSlot(0), s.sourceBlock.location);
		s.sourceBlock.dimension.spawnItem(lock_container.getSlot(1), s.sourceBlock.location);
		lock_container.clearAll();
	}else{
		let target_player = Array.from(s.sourceBlock.dimension.getPlayers({ tags: [ "use_duplicator" ]}))[0];
		let item_hand = target_player.getComponent("minecraft:inventory").container.getItem(target_player.selectedSlot);
		if(item_hand.typeId == "kd:lock_key" && ((s.message == "0" && item_hand.getLore().length > 0 && item_hand.getLore()[0].split(":")[0] == "\n§r§9Lock ID") || (s.message == "1" && item_hand.getLore().length == 0))){
			let loc = s.sourceBlock.location;
			let lock_id = Array.from(s.sourceBlock.dimension.getEntitiesAtlocation(s.sourceBlock.location))[0];
			let lock_container = lock_id.getComponent("minecraft:inventory").container;
			let target_container = target_player.getComponent("minecraft:inventory").container;
			if(s.message == "0"){
				lock_container.setItem(0, item_hand);
			}else if(s.message == "1"){
				lock_container.setItem(1, item_hand);
			}
			let block_set = s.sourceBlock.permutation.clone();
			block_set.getState("kd:key") = s.message == "0" ? 1 : 2;
			s.sourceBlock.setPermutation(block_set);
		}else{
			let duplicate_display = s.message == "0" ? "Select key with Lock ID" : "Select empty key";
			target_player.onScreenDisplay.setActionBar(duplicate_display);
		}
		
		target_player.removeTag("use_duplicator");
	}
}, { namespaces: [ "duplicator" ]}); 
*/